/*--------------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        14.06.2007
 Description:    ADC-Routinen
--------------------------------------------------------------------------------*/
#include "analog.h"

volatile unsigned char adc_on = 1;

volatile int adc0_value_mux[8]  = {0,0,0,0,0,0,0,0};
volatile int adc0_count[8]      = {0,0,0,0,0,0,0,0};
volatile int adc0_neutral[8]    = {0,0,0,0,0,0,0,0};
volatile int adc0_pos[8]        = {0,0,0,1,1,1,0,0};

volatile int adc1_value_mux[8]  = {0,0,0,0,0,0,0,0};
volatile int adc1_count[8]      = {0,0,0,0,0,0,0,0};
volatile int adc1_neutral[8]    = {0,0,0,0,0,0,0,0};
volatile int adc1_pos[8]        = {0,0,0,1,0,0,0,0};

volatile int adc_count[8]       = {0,0,0,0,0,0,0,0};


//
void ADC_Init(void)
{
	//Umschalten auf ADC0 an Pin: P0.25,P0.26,P0.27,P0.28,P0.29,P0.30
	PINSEL1 |= (1<<18)|(1<<20)|(1<<22)|(1<<24)|(1<<26)|(1<<28);
	
	//Umschalten auf ADC1 an Pin: P0.12,P0.22
	PINSEL0 |= (1<<24)|(1<<25);
	PINSEL1 |= (1<<12);
	
	AD1CR |= 0x88; 			//channel 3 und 7
	AD1CR |= (0xFF<<8); 	//Teiler :256
	AD1CR |= (1<<21); 		//PDN
	
	AD0CR |= 0x3F; 			//channel 0,1,2,3,4 und 5
	AD0CR |= (0xFF<<8); 	//Teiler :256
	AD0CR |= (1<<21); 		//PDN
	
	ADGSR |= (1<<16); 		//Globale Burst Bit
	
	adc0_neutral[0] = 276;
	adc0_neutral[1] = 495;
	adc0_neutral[2] = 0;
	adc0_neutral[3] = 537;
	adc0_neutral[4] = 353;
	adc0_neutral[5] = 395;
	
	//set interrupt vector in 2
	VICVectAddr3 = (unsigned long)adc0_int;
	//use it for ADC 0 Interrupt
	VICVectCntl3 = VICVectCntl_ENABLE | VIC_Channel_ADC0; 
	//Enable ADC0 Interrupt
	VICIntEnable = (1<<VIC_Channel_ADC0); 

	//set interrupt vector in 3
	VICVectAddr4 = (unsigned long)adc1_int;
	//use it for ADC 1 Interrupt
	VICVectCntl4 = VICVectCntl_ENABLE | VIC_Channel_ADC1; 
	//Enable ADC1 Interrupt
	VICIntEnable = (1<<VIC_Channel_ADC1);    
}


//-------------------------------------------------------------------------------
//
void __attribute__ ((interrupt("IRQ"))) adc0_int(void) 
{
	unsigned int ADCtmp = AD0DR;
	unsigned char channel = ((ADCtmp>>24)&7); 
    
	if (adc_on)
	{
        adc_count[channel]++;
		if(adc0_pos[channel] == 0)
		{
			adc0_value_mux[channel] += adc0_neutral[channel] - ((ADCtmp>>6)&0x3FF);
		}
		else
		{
			adc0_value_mux[channel] += ((ADCtmp>>6)&0x3FF) - adc0_neutral[channel];
		}
	}
	
	VICVectAddr = 0;       // Acknowledge Interrupt (rough?)
}

//-------------------------------------------------------------------------------
//
void __attribute__ ((interrupt("IRQ"))) adc1_int(void) 
{
	unsigned int ADCtmp = AD1DR;
	unsigned char channel = ((ADCtmp>>24)&7);
	
	if (adc_on)
	{
		if(adc1_pos[channel]==0)
		{
			adc1_value_mux[channel] = adc1_neutral[channel] - ((ADCtmp>>6)&0x3FF);
		}
		else
		{
			adc1_value_mux[channel] = ((ADCtmp>>6)&0x3FF) - adc1_neutral[channel];
		}
	}
	VICVectAddr = 0;       // Acknowledge Interrupt (rough?)
}

void ADC_offset(void)
{	
	unsigned int tmp;
    
    for(tmp = 0;tmp<100000;tmp++){asm("nop");};
    
    ANALOG_OFF;
    adc_count[ADC_NICK] = 0;
    adc_count[ADC_ACCY] = 0;
    adc_count[ADC_ACCX] = 0;
    adc_count[ADC_ROLL] = 0;
    adc_count[ADC_GIER] = 0;

    adc0_value_mux[ADC_NICK] = 0;
    adc0_value_mux[ADC_ACCY] = 0;
    adc0_value_mux[ADC_ACCX] = 0;
    adc0_value_mux[ADC_ROLL] = 0;
    adc0_value_mux[ADC_GIER] = 0;
    ANALOG_ON;
    
	for(tmp = 0;tmp<100000;tmp++){asm("nop");};
    ANALOG_OFF;
	adc0_neutral[ADC_GIER] -= adc0_value_mux[ADC_GIER]/adc_count[ADC_GIER];
	adc0_neutral[ADC_NICK] += adc0_value_mux[ADC_NICK]/adc_count[ADC_NICK];
	adc0_neutral[ADC_ROLL] += adc0_value_mux[ADC_ROLL]/adc_count[ADC_ROLL];
	adc0_neutral[ADC_ACCX] -= adc0_value_mux[ADC_ACCX]/adc_count[ADC_ACCX];
	adc0_neutral[ADC_ACCY] += adc0_value_mux[ADC_ACCY]/adc_count[ADC_ACCY];
	//adc0_neutral[ADC_ACCZ] -= adc0_mittelwert[ADC_ACCZ];
    ANALOG_ON;
}

