/*--------------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        18.05.2008
 Description:    PID and Mixer 
--------------------------------------------------------------------------------*/
#include "fc.h"

volatile signed int y_old = 0;
volatile signed int y_sum = 0;
volatile signed int y_e_sum = 0;
volatile signed int y_rc_mittelwert = 0;

volatile signed int x_old = 0;
volatile signed int x_sum = 0;
volatile signed int x_e_sum = 0;
volatile signed int x_rc_mittelwert = 0;

volatile signed int gier_old = 0;

//-------------------------------------------------------------------------------
//
void flight_control (void)
{
    ANALOG_OFF;
    
	signed int gier = -(adc0_value_mux[ADC_GIER]/adc_count[ADC_GIER]);
    
    signed int y_out = 0;
    signed int y_gyro_error = 0;
    signed int y_b = adc0_value_mux[ADC_ACCY]/adc_count[ADC_ACCY];
    signed int y_g = adc0_value_mux[ADC_NICK]/adc_count[ADC_NICK];
    
    signed int x_out = 0;
    signed int x_gyro_error = 0;
    signed int x_b = adc0_value_mux[ADC_ACCX]/adc_count[ADC_ACCX];
    signed int x_g = adc0_value_mux[ADC_ROLL]/adc_count[ADC_ROLL];
	
	signed int e, w, rc_error = 0;
    signed int motor_value = 10;
    
    signed int tmp;
    
    adc_count[ADC_NICK] = 0;
    adc_count[ADC_ACCY] = 0;
    adc_count[ADC_ACCX] = 0;
    adc_count[ADC_ROLL] = 0;
    adc_count[ADC_GIER] = 0;
    
    adc0_value_mux[ADC_NICK] = 0;
    adc0_value_mux[ADC_ACCY] = 0;
    adc0_value_mux[ADC_ACCX] = 0;
    adc0_value_mux[ADC_ROLL] = 0;
    adc0_value_mux[ADC_GIER] = 0;
  
    ANALOG_ON;

	//---------------------------------------------------------------------------
	//PD Gyro (GIER)
    
    //e = w - x (Sollwert = 0)
    e = (gier * 0.5) - (rc_ch[RC_GIER] * 0.4);
    
    //P-Regelpart Kp * e
    gier = 4 * e;
    
    //D-Regelpart Kd/ta * e (Zeit ta = 0,01s)
    gier += 10 * (e - gier_old);
    gier_old = e;
        
	//---------------------------------------------------------------------------
	//Gyro Signal integrieren und an AAC abgleichen
    rc_error = rc_ch[RC_NICK] - y_rc_mittelwert;
    
    if (abs(rc_error) > 20)
    {
        if (rc_ch[RC_NICK] > y_rc_mittelwert) y_rc_mittelwert += 3;
        if (rc_ch[RC_NICK] < y_rc_mittelwert) y_rc_mittelwert -= 3;
    }
    else
    {
        if (rc_ch[RC_NICK] > y_rc_mittelwert) y_rc_mittelwert++;
        if (rc_ch[RC_NICK] < y_rc_mittelwert) y_rc_mittelwert--; 
    }
 
    y_sum += y_g;
    
    y_gyro_error = y_sum - (y_b * 70);
    
    if(y_gyro_error > 3000) y_gyro_error = 3000;
    if(y_gyro_error < -3000) y_gyro_error =-3000;
    
    y_sum -= y_gyro_error * 0.005;
 
    //----------------------------------------
    //PI - Regler
   
    //e = w - x (Sollwert = 0)
    w = (y_g * 0.3) + (y_sum * 0.003);
      
    e = w - (y_rc_mittelwert*0.2);

    //Integral bilden und begrenzen (I-Anteil)
    y_e_sum += e; 
    if (y_e_sum > 3000) y_e_sum = 3000;
    if (y_e_sum < -3000) y_e_sum = -3000;

    //P-Regler Kp * e + Ki + e_sum_nick1 (P + I-Anteil)
    y_out = 3 * e + (y_e_sum*0.025);
    
    //D-Regelpart Kd/ta * e (Zeit ta = 0,01s) (+D Anteil)
    //y_out += 2 * (e - y_old);
    y_old = e;
    
    //----------------------------------------
    //Werte mischen
  
	motor_value = rc_ch[RC_GAS] + 70 + y_out + gier;	
	if (motor_value < 5) motor_value = 5;
	if (motor_value > 255) motor_value = 255;
	PWM_Motor[0]= motor_value;

	motor_value = rc_ch[RC_GAS] + 70 - y_out + gier;	
	if (motor_value < 5) motor_value = 5;	
	if (motor_value > 255) motor_value = 255;
	PWM_Motor[1]= motor_value;

	//---------------------------------------------------------------------------
	//Gyro Signal integrieren und an AAC abgleichen	
    rc_error = rc_ch[RC_ROLL] - x_rc_mittelwert;
    
    if (abs(rc_error) > 20)
    {
        if (rc_ch[RC_ROLL] > x_rc_mittelwert) x_rc_mittelwert += 3;
        if (rc_ch[RC_ROLL] < x_rc_mittelwert) x_rc_mittelwert -= 3;
    }
    else
    {
        if (rc_ch[RC_ROLL] > x_rc_mittelwert) x_rc_mittelwert++;
        if (rc_ch[RC_ROLL] < x_rc_mittelwert) x_rc_mittelwert--; 
    }
 
    x_sum += x_g;
    
    x_gyro_error = x_sum - (x_b * 80);
    
    if(x_gyro_error > 4000) x_gyro_error = 4000;
    if(x_gyro_error < -4000) x_gyro_error =-4000;
    
    x_sum -= x_gyro_error * 0.005;
    
    tmp = x_gyro_error* 0.005;
    
    print_f("GS:%4i  -  GE:%4i\r\n",y_sum,tmp);
 
    //----------------------------------------
    //PI - Regler
     
    //e = w - x (Sollwert = 0)
    w = (x_g * 0.3) + (x_sum*0.003);
      
    e = w - (x_rc_mittelwert*0.2);

    //Integral bilden und begrenzen (I-Anteil)
    x_e_sum += e; 
    if (x_e_sum > 3000) x_e_sum = 3000;
    if (x_e_sum < -3000) x_e_sum = -3000;

    //P-Regler Kp * e + Ki + e_sum_nick1 (P + I-Anteil)
    x_out = 3 * e + (x_e_sum*0.025);
    
    //D-Regelpart Kd/ta * e (Zeit ta = 0,01s) (+D Anteil)
    //x_out += 2 * (e - x_old);
    x_old = e;
   
    //----------------------------------------
    //Werte mischen
 
	motor_value = rc_ch[RC_GAS] + 70 - x_out - gier;	
	if (motor_value < 10) motor_value = 10;
	if (motor_value > 255) motor_value = 255;
	PWM_Motor[2]= motor_value;

	motor_value = rc_ch[RC_GAS] + 70 + x_out - gier;	
	if (motor_value < 10) motor_value = 10;	
	if (motor_value > 255) motor_value = 255;
	PWM_Motor[3]= motor_value;
}

    

 
