/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        14.06.2007
 Description:    Hauptprogramm BL_UFO
------------------------------------------------------------------------------*/

#include <lpc213x.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"
#include "uart.h"
#include "printf_P.h"
#include "viclowlevel.h"
#include "typedefs.h"
#include "clock-arch.h"
#include "analog.h"
#include "rc.h"
#include "twimaster.h"
#include "fc.h"

void SystemInit(void);

//-------------------------------------------------------------------------------
//Init System PLL (60Mhz)
void SystemInit(void)
{
	// --- enable and connect the PLL (Phase Locked Loop) ---
	// a. set multiplier and divider
	PLLCFG = MSEL | (1<<PSEL1) | (0<<PSEL0);
	// b. enable PLL
	PLLCON = (1<<PLLE);
	// c. feed sequence
	PLLFEED = PLL_FEED1;
	PLLFEED = PLL_FEED2;
	// d. wait for PLL lock (PLOCK bit is set if locked)
	while (!(PLLSTAT & (1<<PLOCK)));
	// e. connect (and enable) PLL
	PLLCON = (1<<PLLE) | (1<<PLLC);
	// f. feed sequence
	PLLFEED = PLL_FEED1;
	PLLFEED = PLL_FEED2;
	
	// --- setup and enable the MAM (Memory Accelerator Module) ---
	// a. start change by turning of the MAM (redundant)
	MAMCR = 0;	
	// b. set MAM-Fetch cycle to 3 cclk as recommended for >40MHz
	MAMTIM = MAM_FETCH;
	// c. enable MAM 
	MAMCR = MAM_MODE;
	
	// --- set VPB speed ---
	VPBDIV = VPBDIV_VAL;
	
	// --- map INT-vector ---
	#if defined(RAM_RUN)
	  MEMMAP = MEMMAP_USER_RAM_MODE;
	#elif defined(ROM_RUN)
	  MEMMAP = MEMMAP_USER_FLASH_MODE;
	#else
	#error RUN_MODE not defined!
	#endif
}

//-------------------------------------------------------------------------------
// Hauptprogramm BL - UFO
int main(void)
{
	unsigned int tmp;
    IODIR1 |= (1<<21)|(1<<22);//Led rot und Led grn (Port Output)
	IODIR1 |= (1<<27);//Speaker Output
	ROT_ON;

    VICIntEnClear = 0xFFFFFFFF;
    VICIntSelect = 0x00000000; 

	SystemInit();
	
	enableIRQ(); 
		
	i2c_init();	
	uart0Init(B38400, UART_8N1, UART_FIFO_8); // setup the UART	
    init_timer();
    rc_sum_init();
   	ADC_Init();

	speaker_on = 8;
	print_f("\n\rARM BL-UFO V0.01 \n\r");
	for(tmp = 0;tmp<1000000;tmp++){asm("nop");};
	ADC_offset();
	
	ROT_OFF;
	GRN_ON;
	while((rc_ch[RC_GAS]>-80)|(rc_valid < 100)); //Wartet bis RC-Gas aus!!!!
	speaker_on = 0;

	while(1){	
		//Start Flight
        
        
		while((rc_ch[RC_GAS]<-80) || (rc_valid < 100)) //Wartet bis RC-Gas an!!!!
		{
            print_f("Nick:%4i; Roll:%4i; R:%4i; X:%4i; Y:%4i; Z:%4i; \r\n",rc_ch[RC_NICK],
                                                                rc_ch[RC_ROLL],
                                                                (adc0_value_mux[ADC_NICK]/adc_count[ADC_NICK]),
                                                                (adc0_value_mux[ADC_ACCX]/adc_count[ADC_ACCX]),
                                                                (adc0_value_mux[ADC_ACCY]/adc_count[ADC_ACCY]),
                                                                (adc0_value_mux[ADC_ACCZ]/adc_count[ADC_ACCZ]));
                                                                
            adc_count[ADC_NICK] = 0;
            adc_count[ADC_ACCY] = 0;
            adc_count[ADC_ACCX] = 0;
            adc_count[ADC_ROLL] = 0;
            adc_count[ADC_GIER] = 0;
        
            adc0_value_mux[ADC_NICK] = 0;
            adc0_value_mux[ADC_ACCY] = 0;
            adc0_value_mux[ADC_ACCX] = 0;
            adc0_value_mux[ADC_ROLL] = 0;
            adc0_value_mux[ADC_GIER] = 0;
		}
        
        
		while ((rc_ch[RC_GAS]>-80) || (rc_valid < 100)) //Fliegen bis RC-Gas aus!!!!
        { 
			if(timer_counter > 120) 
			{
				timer_counter = 0;
				flight_control();
			}
		}
			motor_off(); 
			ROT_ON;
	}	
	
	return (1);
}








