/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        14.06.2007
 Description:    I2C Routinen
------------------------------------------------------------------------------*/
#include "twimaster.h"

unsigned char twi_state = 0;
unsigned char motor = 0;
volatile unsigned char PWM_Motor[4] = {0,0,0,0};

//-------------------------------------------------------------------------------
//Initzialisieren der I2C (TWI) Schnittstelle
void i2c_init(void)
{
	//Umschalten auf I2C0 an Pin: P0.02,P0.03
	PINSEL0 |= (1<<4)|(1<<6);
	
	I2C0SCLH=0x01FF;
	I2C0SCLL=0x01FF;
	
	I2C0CONSET |= I2EN; //I2C Enable
	
	I2C0CONCLR=0x6c; //Clear all Flags

 	//set interrupt vector in 1
	VICVectAddr0 = (unsigned long)i2c_int;
	
	//use it for Timer 0 Interrupt
	VICVectCntl0 = VICVectCntl_ENABLE | VIC_Channel_I2C0; 
	
	//Enable Timer0 Interrupt
	VICIntEnable = (1<<VIC_Channel_I2C0); 
	
	motor_off(); 	
	i2c_start();   	
}

//-------------------------------------------------------------------------------
//Start I2C
void motor_off(void) 
{
	PWM_Motor[0]= 0;			
	PWM_Motor[1]= 0;
	PWM_Motor[2]= 0;			
	PWM_Motor[3]= 0; 
}
//-------------------------------------------------------------------------------
//Start I2C
char i2c_start(void) 
{
	I2C0CONCLR=0x6c; //Clear all Flags
	I2C0CONSET = STA | I2EN;
    return(0);
}

//-------------------------------------------------------------------------------
//Stop I2C
void i2c_stop(void)
{
	I2C0CONSET=STO;
	I2C0CONCLR=0x6c; //Clear all Flags
}

//-------------------------------------------------------------------------------
//I2C write byte
char i2c_write_byte(char byte)
{ 
	I2C0DAT = byte;
	I2C0CONCLR=0x28;
    return(0);  
}

//-------------------------------------------------------------------------------
//Start I2C
void __attribute__ ((interrupt("IRQ"))) i2c_int(void) 
{
    switch (twi_state++)
        {
        case 0:
                i2c_write_byte(0x52+(motor*2));
                break;
        case 1:
                i2c_write_byte(PWM_Motor[motor++]);
                break;
        case 2:
                i2c_stop();
                if (motor==4)
				{
					motor = 0;
				}
				twi_state = 0; 
				i2c_start(); 
                break; 
        }
	VICVectAddr = 0;
}

